#!/usr/bin/env python

"""Module containing the Mmcif class and the command line interface."""

from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import (
    check_mandatory_property,
    check_output_path,
    download_mmcif,
    write_mmcif,
)


class Mmcif(BiobbObject):
    """
    | biobb_io Mmcif
    | This class is a wrapper for downloading a MMCIF structure from the Protein Data Bank.
    | Wrapper for the `Protein Data Bank in Europe <https://www.ebi.ac.uk/pdbe/>`_, the `Protein Data Bank <https://www.rcsb.org/>`_ and the `MMB PDB mirror <https://mdb-login.bsc.es/api/>`_ for downloading a single MMCIF structure.

    Args:
        output_mmcif_path (str): Path to the output MMCIF file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/ref_output.mmcif>`_. Accepted formats: cif (edam:format_1477), mmcif (edam:format_1477).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB code.
            * **api_id** (*str*) - ("pdbe") Identifier of the PDB REST API from which the MMCIF structure will be downloaded. Values: pdbe (`PDB in Europe REST API <https://www.ebi.ac.uk/pdbe/pdbe-rest-api>`_), pdb (`RCSB PDB REST API <https://data.rcsb.org/>`_), mmb (`MMB PDB mirror API <https://mdb-login.bsc.es/api/>`_).
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.mmcif import mmcif
            prop = {
                'pdb_code': '2VGB',
                'api_id': 'pdbe'
            }
            mmcif(output_mmcif_path='/path/to/newStructure.mmcif',
                properties=prop)

    Info:
        * wrapped_software:
            * name: Protein Data Bank
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_mmcif_path, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_mmcif_path": output_mmcif_path}}

        # Properties specific for BB
        self.api_id = properties.get("api_id", "pdbe")
        self.pdb_code = properties.get("pdb_code", None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_mmcif_path = check_output_path(
            self.io_dict["out"]["output_mmcif_path"],
            "output_mmcif_path",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Mmcif <api.mmcif.Mmcif>` api.mmcif.Mmcif object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        check_mandatory_property(
            self.pdb_code, "pdb_code", self.out_log, self.__class__.__name__
        )

        self.pdb_code = self.pdb_code.strip().lower()

        # Downloading PDB file
        mmcif_string = download_mmcif(
            self.pdb_code, self.api_id, self.out_log, self.global_log
        )
        write_mmcif(mmcif_string, self.output_mmcif_path, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def mmcif(output_mmcif_path: str, properties: Optional[dict] = None, **kwargs) -> int:
    """Execute the :class:`Mmcif <api.mmcif.Mmcif>` class and
    execute the :meth:`launch() <api.mmcif.Mmcif.launch>` method."""
    return Mmcif(**dict(locals())).launch()


mmcif.__doc__ = Mmcif.__doc__
main = Mmcif.get_main(mmcif, "This class is a wrapper for downloading a MMCIF structure from the Protein Data Bank.")

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""Module containing the MDDB class and the command line interface."""

from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import (
    check_mandatory_property,
    check_output_path,
    download_mddb_top,
    write_pdb,
    download_mddb_trj,
    write_bin,
    download_mddb_file
)


class MDDB(BiobbObject):
    """
    | biobb_io MDDB
    | This class is a wrapper for downloading a trajectory / topology pair from the MDDB Database.
    | Wrapper for the `MDDB Database <https://mmb.mddbr.eu/>`_ for downloading a trajectory and its corresponding topology.

    Args:
        output_top_path (str): Path to the output toplogy file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_mddb.pdb>`_. Accepted formats: pdb (edam:format_1476).
        output_trj_path (str): Path to the output trajectory file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_mddb.xtc>`_. Accepted formats: mdcrd (edam:format_3878), trr (edam:format_3910), xtc (edam:format_3875).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **project_id** (*str*) - (None) Project accession or identifier.
            * **node_id** (*str*) - ("mmb") MDDB node identifier.
            * **trj_format** (*str*) - ("xtc") Trajectory format. Values: mdcrd (AMBER trajectory format), trr (Trajectory of a simulation experiment used by GROMACS), xtc (Portable binary format for trajectories produced by GROMACS package).
            * **frames** (*str*) - (None) Specify a frame range for the returned trajectory. Ranges are defined by dashes, and multiple ranges can be defined separated by commas. It can also be defined as the start:end:step format (ie: '10:20:2').
            * **selection** (*str*) - (None) Specify a NGL-formatted selection for the returned trajectory. See here for the kind of selection that can be used: http://nglviewer.org/ngl/api/manual/usage/selection-language.html.
            * **extra_files** (*list*) - (None) List of extra files to download. It should be a tuple with the name of the file and the path to be saved.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.mddb import mddb
            prop = {
                'project_id': 'A0001',
                'trj_format': 'xtc'
            }
            mddb(output_top_path='/path/to/newTopology.pdb',
                 output_trj_path='/path/to/newTrajectory.pdb',
                 properties=prop)

    Info:
        * wrapped_software:
            * name: MDDB Database
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_top_path, output_trj_path, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_top_path": output_top_path, "output_trj_path": output_trj_path}}

        # Properties specific for BB
        self.project_id = properties.get("project_id", None)
        self.node_id = properties.get("node_id", "mmb")
        self.trj_format = properties.get("trj_format", "xtc")
        self.frames = properties.get("frames", "")
        self.selection = properties.get("selection", "*")
        self.extra_files = properties.get("extra_files", [])
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_top_path = check_output_path(
            self.io_dict["out"]["output_top_path"],
            "output_top_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.output_trj_path = check_output_path(
            self.io_dict["out"]["output_trj_path"],
            "output_trj_path",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`MDDB <api.mddb.MDDB>` api.mddb.MDDB object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        check_mandatory_property(
            self.project_id, "project_id", self.out_log, self.__class__.__name__
        )

        self.project_id = self.project_id.strip().upper()

        # Downloading topology file
        top_string = download_mddb_top(
            self.project_id,
            self.node_id,
            self.selection,
            self.out_log,
            self.global_log,
            self.__class__.__name__
        )
        write_pdb(top_string, self.output_top_path, None, self.out_log, self.global_log)

        # Downloading trajectory file
        trj_string = download_mddb_trj(
            self.project_id,
            self.node_id,
            self.trj_format,
            self.frames,
            self.selection,
            self.out_log,
            self.global_log,
            self.__class__.__name__,
        )
        write_bin(trj_string, self.output_trj_path, self.out_log, self.global_log)

        for (extra_file, extra_path) in self.extra_files:
            try:
                file_string = download_mddb_file(
                    self.project_id,
                    self.node_id,
                    extra_file,
                    self.out_log,
                    self.global_log,
                    self.__class__.__name__,
                )
                write_bin(file_string, extra_path, self.out_log, self.global_log)
            except Exception:
                pass

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def mddb(output_top_path: str, output_trj_path: str, properties: Optional[dict] = None, **kwargs) -> int:
    """Execute the :class:`MDDB <api.mddb.MDDB>` class and
    execute the :meth:`launch() <api.mddb.MDDB.launch>` method."""
    return MDDB(**dict(locals())).launch()


mddb.__doc__ = MDDB.__doc__
main = MDDB.get_main(
    mddb,
    "This class is a wrapper for downloading a trajectory / topology pair from the MDDB Database.",
    custom_flags={"output_top_path": "-o", "output_trj_path": "-t"}
)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""Module containing the StructureInfo class and the command line interface."""

from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import (
    check_mandatory_property,
    check_output_path,
    download_str_info,
    write_json,
)


class StructureInfo(BiobbObject):
    """
    | biobb_io StructureInfo
    | This class is a wrapper for getting all the available information of a structure from the Protein Data Bank.
    | Wrapper for the `MMB PDB mirror <https://mdb-login.bsc.es/api/>`_ for getting all the available information of a structure from the Protein Data Bank.

    Args:
        output_json_path (str): Path to the output JSON file with all the structure information. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/ref_str_info.json>`_. Accepted formats: json (edam:format_3464).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB structure code.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.structure_info import structure_info
            prop = {
                'pdb_code': '2vgb'
            }
            structure_info(output_json_path='/path/to/newStructure.sdf',
                        properties=prop)

    Info:
        * wrapped_software:
            * name: Protein Data Bank
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_json_path, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_json_path": output_json_path}}

        # Properties specific for BB
        self.pdb_code = properties.get("pdb_code", None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_json_path = check_output_path(
            self.io_dict["out"]["output_json_path"],
            "output_json_path",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`StructureInfo <api.structure_info.StructureInfo>` api.structure_info.StructureInfo object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        check_mandatory_property(
            self.pdb_code, "pdb_code", self.out_log, self.__class__.__name__
        )

        self.pdb_code = self.pdb_code.strip().lower()
        url = "https://mdb-login.bsc.es/api/pdb/%s.json"

        # Downloading PDB file
        json_string = download_str_info(
            self.pdb_code, url, self.out_log, self.global_log
        )
        write_json(json_string, self.output_json_path, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def structure_info(
    output_json_path: str, properties: Optional[dict] = None, **kwargs
) -> int:
    """Execute the :class:`StructureInfo <api.structure_info.StructureInfo>` class and
    execute the :meth:`launch() <api.structure_info.StructureInfo.launch>` method."""
    return StructureInfo(**dict(locals())).launch()


structure_info.__doc__ = StructureInfo.__doc__
main = StructureInfo.get_main(structure_info, "This class is a wrapper for getting all the available information of a structure from the Protein Data Bank.")

if __name__ == "__main__":
    main()



"""Common functions for package api"""

import json
import os
import re
import urllib.request
import urllib.parse
from pathlib import Path, PurePath

import requests
from biobb_common.tools import file_utils as fu


def check_output_path(path, argument, optional, out_log, classname) -> str:
    """Checks output file"""
    if optional and not path:
        return ""
    if PurePath(path).parent and not Path(PurePath(path).parent).exists():
        fu.log(classname + ": Unexisting %s folder, exiting" % argument, out_log)
        raise SystemExit(classname + ": Unexisting %s folder" % argument)
    file_extension = PurePath(path).suffix
    if not is_valid_file(file_extension[1:], argument):
        fu.log(
            classname + ": Format %s in %s file is not compatible"
            % (file_extension[1:], argument),
            out_log,
        )
        raise SystemExit(
            classname + ": Format %s in %s file is not compatible"
            % (file_extension[1:], argument)
        )
    return path


def is_valid_file(ext, argument):
    """Checks if file format is compatible"""
    formats = {
        "output_sdf_path": ["sdf"],
        "output_pdb_path": ["pdb"],
        "output_simulations": ["json"],
        "output_simulation": ["zip"],
        "output_pdb_zip_path": ["zip"],
        "output_mutations_list_txt": ["txt"],
        "output_json_path": ["json"],
        "output_fasta_path": ["fasta"],
        "output_mmcif_path": ["mmcif", "cif"],
        "output_top_path": ["pdb"],
        "output_trj_path": ["mdcrd", "trr", "xtc"]
    }
    return ext in formats[argument]


def download_pdb(pdb_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the pdb file.
    """

    if api_id == "mmb":
        url = "https://mdb-login.bsc.es/api/pdb/" + pdb_code + "/coords/?"
    elif api_id == "pdb":
        url = "https://files.rcsb.org/download/" + pdb_code + ".pdb"
    elif api_id == "pdbe":
        url = "https://www.ebi.ac.uk/pdbe/entry-files/download/pdb" + pdb_code + ".ent"

    fu.log("Downloading %s from: %s" % (pdb_code, url), out_log, global_log)
    return requests.get(url).content.decode("utf-8")


def download_af(uniprot_code, out_log=None, global_log=None, classname=None):
    """
    Returns:
        String: Content of the pdb file.
    """

    url = "https://alphafold.ebi.ac.uk/files/AF-" + uniprot_code + "-F1-model_v6.pdb"

    fu.log("Downloading %s from: %s" % (uniprot_code, url), out_log, global_log)

    r = requests.get(url)
    if r.status_code == 404:
        fu.log(classname + ": Incorrect Uniprot Code: %s" % (uniprot_code), out_log)
        raise SystemExit(classname + ": Incorrect Uniprot Code: %s" % (uniprot_code))

    return r.content.decode("utf-8")


def download_mddb_top(project_id, node_id, selection, out_log=None, global_log=None, classname=None):
    """
    Returns:
        String: Content of the pdb file.
    """

    url = "https://" + node_id + ".mddbr.eu/api/rest/v1/projects/" + project_id + "/structure?selection=" + urllib.parse.quote(str(selection))

    fu.log("Downloading %s topology from: %s" % (project_id, url), out_log, global_log)

    r = requests.get(url)
    if r.status_code == 404:
        fu.log(classname + ": Incorrect url, check project_id, node_id and selection: %s" % (url), out_log)
        raise SystemExit(classname + ": Incorrect url, check project_id, node_id and selection: %s" % (url))

    return r.content.decode("utf-8")


def download_mddb_trj(project_id, node_id, trj_format, frames, selection, out_log=None, global_log=None, classname=None):
    """
    Returns:
        String: Content of the trajectory file.
    """

    url = "https://" + node_id + ".mddbr.eu/api/rest/v1/projects/" + project_id + "/trajectory?format=" + trj_format + "&frames=" + frames + "&selection=" + urllib.parse.quote(str(selection))

    fu.log("Downloading %s trajectory from: %s" % (project_id, url), out_log, global_log)

    r = requests.get(url)
    if r.status_code == 404:
        fu.log(classname + ": Incorrect url, check project_id, node_id, trj_format, frames and selection: %s" % (url), out_log)
        raise SystemExit(classname + ": Incorrect url, check project_id, node_id, trj_format, frames and selection: %s" % (url))

    return r.content


def download_mddb_file(project_id, node_id, file_name, out_log=None, global_log=None, classname=None):
    """
    Returns:
        String: Content of the trajectory file.
    """

    url = "https://" + node_id + ".mddbr.eu/api/rest/v1/projects/" + project_id + "/files/" + file_name

    fu.log("Downloading %s file from: %s" % (project_id, url), out_log, global_log)

    r = requests.get(url)
    if r.status_code == 404:
        fu.log(classname + ": Incorrect url, check project_id, node_id, trj_format, frames and selection: %s" % (url), out_log)
        raise SystemExit(classname + ": Incorrect url, check project_id, node_id, trj_format, frames and selection: %s" % (url))

    return r.content


def download_mmcif(pdb_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the mmcif file.
    """

    if api_id == "mmb":
        url = "https://mdb-login.bsc.es/api/pdb/" + pdb_code + ".cif"
    elif api_id == "pdb":
        url = "https://files.rcsb.org/download/" + pdb_code + ".cif"
    elif api_id == "pdbe":
        url = "https://www.ebi.ac.uk/pdbe/entry-files/download/" + pdb_code + ".cif"

    fu.log("Downloading %s from: %s" % (pdb_code, url), out_log, global_log)
    return requests.get(url, verify=True).content.decode("utf-8")


def download_ligand(ligand_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the ligand file.
    """

    if api_id == "mmb":
        url = "https://mdb-login.bsc.es/api/pdbMonomer/" + ligand_code.lower()
        text = requests.get(url, verify=True).content.decode("utf-8")
    elif api_id == "pdbe":
        url = (
            # "https://www.ebi.ac.uk/pdbe/static/files/pdbechem_v2/" + ligand_code.upper() + "_ideal.pdb"
            "https://files.rcsb.org/ligands/view/" + ligand_code.upper() + ".cif"
        )
        text = urllib.request.urlopen(url).read().decode("utf-8")

    fu.log("Downloading %s from: %s" % (ligand_code, url), out_log, global_log)

    # removing useless empty lines at the end of the file
    text = os.linesep.join([s for s in text.splitlines() if s])

    return text


def download_fasta(pdb_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the fasta file.
    """

    if api_id == "mmb":
        url = "https://mdb-login.bsc.es/api/pdb/" + pdb_code + ".fasta"
    elif api_id == "pdb":
        url = "https://www.rcsb.org/fasta/entry/" + pdb_code
    elif api_id == "pdbe":
        url = "https://www.ebi.ac.uk/pdbe/api/v2/pdb/entry/" + pdb_code + "/fasta"

    fu.log("Downloading %s from: %s" % (pdb_code, url), out_log, global_log)
    return requests.get(url, verify=True).content.decode("utf-8")


def download_binding_site(
    pdb_code,
    url="https://www.ebi.ac.uk/pdbe/api/v2/pdb/entry/binding_sites/%s/1",
    out_log=None,
    global_log=None,
):
    """
    Returns:
        String: Content of the component file.
    """
    url = url % pdb_code

    fu.log("Getting binding sites from: %s" % (url), out_log, global_log)

    text = urllib.request.urlopen(url).read()
    json_obj = json.loads(text)
    json_string = json.dumps(json_obj, indent=4, sort_keys=True)
    # json_string = json.dumps(text, indent=4)

    return json_string


def download_ideal_sdf(ligand_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the ideal sdf file.
    """

    if api_id == "pdb":
        url = (
            "https://files.rcsb.org/ligands/download/" + ligand_code.upper() + "_ideal.sdf"
        )
        text = requests.get(url, verify=True).content.decode("utf-8")
    elif api_id == "pdbe":
        url = (
            "https://files.wwpdb.org/pub/pdb/refdata/chem_comp/" + ligand_code.upper()[-1] + "/" + ligand_code.upper() + "/" + ligand_code.upper() + "_ideal.sdf"
        )
        text = urllib.request.urlopen(url).read().decode("utf-8")

    fu.log("Downloading %s from: %s" % (ligand_code, url), out_log, global_log)

    return text


def download_str_info(
    pdb_code,
    url="https://mdb-login.bsc.es/api/pdb/%s.json",
    out_log=None,
    global_log=None,
):
    """
    Returns:
        String: Content of the JSON file.
    """
    url = url % pdb_code

    fu.log("Getting structure info from: %s" % (url), out_log, global_log)

    text = urllib.request.urlopen(url).read()
    json_obj = json.loads(text)
    json_string = json.dumps(json_obj, indent=4, sort_keys=True)
    # json_string = json.dumps(text, indent=4)

    return json_string


def write_pdb(pdb_string, output_pdb_path, filt=None, out_log=None, global_log=None):
    """Writes and filters a PDB"""
    fu.log("Writting pdb to: %s" % (output_pdb_path), out_log, global_log)
    with open(output_pdb_path, "w") as output_pdb_file:
        if filt:
            fu.log(
                "Filtering lines NOT starting with one of these words: %s" % str(filt),
                out_log,
                global_log,
            )
            for line in pdb_string.splitlines(True):
                if line.strip().split()[0][0:6] in filt:
                    output_pdb_file.write(line)
        else:
            output_pdb_file.write(pdb_string)


def write_bin(bin_string, output_bin_path, out_log=None, global_log=None):
    """Writes a BIN"""
    fu.log("Writting bin to: %s" % (output_bin_path), out_log, global_log)
    with open(output_bin_path, "wb") as output_bin_file:
        output_bin_file.write(bin_string)


def write_mmcif(mmcif_string, output_mmcif_path, out_log=None, global_log=None):
    """Writes a mmcif"""
    fu.log("Writting mmcif to: %s" % (output_mmcif_path), out_log, global_log)
    with open(output_mmcif_path, "w") as output_mmcif_file:
        output_mmcif_file.write(mmcif_string)


def write_fasta(fasta_string, output_fasta_path, out_log=None, global_log=None):
    """Writes a FASTA"""
    fu.log("Writting FASTA to: %s" % (output_fasta_path), out_log, global_log)
    with open(output_fasta_path, "w") as output_fasta_file:
        output_fasta_file.write(fasta_string)


def write_sdf(sdf_string, output_sdf_path, out_log=None, global_log=None):
    """Writes a SDF"""
    fu.log("Writting sdf to: %s" % (output_sdf_path), out_log, global_log)
    with open(output_sdf_path, "w") as output_sdf_file:
        output_sdf_file.write(sdf_string)


def get_cluster_pdb_codes(pdb_code, cluster, out_log=None, global_log=None):
    """
    Returns:
        String list: The list of pdb_codes of the selected cluster.
    """
    url = "https://mdb-login.bsc.es/api/pdb/"
    pdb_codes = set()

    url = url + pdb_code.lower() + "/clusters/cl-" + str(cluster) + ".json"
    cluster_list = json.loads(requests.get(url, verify=True).content.decode("utf-8"))[
        "clusterMembers"
    ]
    for elem in cluster_list:
        pdb_codes.add(elem["_id"].lower())

    if out_log:
        out_log.info(
            "Cluster: " + str(cluster) + " of pdb_code: " + pdb_code + "\n List: " + str(pdb_codes)
        )
    if global_log:
        global_log.info(
            fu.get_logs_prefix() + "Cluster: " + str(cluster) + " of pdb_code: " + pdb_code + "\n List: " + str(pdb_codes)
        )

    return pdb_codes


def get_uniprot(pdb_code, url, out_log=None, global_log=None):
    """Returns the UNIPROT code corresponding to the `pdb_code`.

    Returns:
        str: UNIPROT code.
    """
    url_uniprot_id = url + "/pdb/" + pdb_code.lower() + "/entry/uniprotRefs/_id"
    uniprot_id = requests.get(url_uniprot_id, verify=True).json()["uniprotRefs._id"][0]

    if out_log:
        out_log.info(
            "PDB code: " + pdb_code + " correspond to uniprot id: " + uniprot_id
        )
    if global_log:
        global_log.info(
            "PDB code: " + pdb_code + " correspond to uniprot id: " + uniprot_id
        )

    return uniprot_id


def get_variants(
    uniprot_id, url="https://mdb-login.bsc.es/api", out_log=None, global_log=None
):
    """Returns the variants of the `uniprot_id` code.

    Returns:
        :obj:`list` of :obj:`str`: List of variants.
    """
    url_uniprot_mut = (
        url + "/uniprot/" + uniprot_id + "/entry/variants/vardata/mut/?varorig=humsavar"
    )
    variants = requests.get(url_uniprot_mut, verify=True).json()["variants.vardata.mut"]
    variants = variants if variants else []

    fu.log(
        "Found: %d variants for uniprot id: %s" % (len(variants), uniprot_id),
        out_log,
        global_log,
    )
    return variants if variants else []


def write_json(json_string, output_json_path, out_log=None, global_log=None):
    """Writes a JSON"""
    fu.log("Writting json to: %s" % (output_json_path), out_log, global_log)
    with open(output_json_path, "w") as output_json_file:
        output_json_file.write(json_string)


def get_memprotmd_sim_list(out_log=None, global_log=None):
    """Returns all available membrane-protein systems (simulations) from the MemProtMD DB using its REST API"""

    fu.log(
        "Getting all available membrane-protein systems (simulations) from the MemProtMD REST API",
        out_log,
        global_log,
    )

    url = "http://memprotmd.bioch.ox.ac.uk/api/simulations/all"
    json_obj = requests.post(url).json()
    json_string = json.dumps(json_obj, indent=4)

    fu.log("Total number of simulations: %d" % (len(json_obj)), out_log, global_log)

    return json_string


def get_memprotmd_sim_search(collection_name, keyword, out_log=None, global_log=None):
    """Performs advanced searches in the MemProtMD DB using its REST API and a given keyword"""

    fu.log(
        "Getting search results from the MemProtMD REST API. Collection name: %s, keyword: %s"
        % (collection_name, keyword),
        out_log,
        global_log,
    )

    url = "http://memprotmd.bioch.ox.ac.uk/api/search/advanced"
    json_query = {
        "collectionName": collection_name,
        "query": {"keywords": keyword},
        "projection": {"simulations": 1},
        "options": {},
    }

    json_obj = requests.post(url, json=json_query).json()
    json_string = json.dumps(json_obj, indent=4)

    # get total number of simulation
    list_kw = []
    for sim_list in json_obj:
        for sim in sim_list["simulations"]:
            list_kw.append(sim)

    fu.log("Total number of simulations: %d" % (len(list_kw)), out_log, global_log)

    return json_string


def get_memprotmd_sim(pdb_code, output_file, out_log=None, global_log=None):
    """Gets a single simulation from MemProtMD DB"""

    fu.log("Getting simulation file from pdb code %s" % (pdb_code), out_log, global_log)

    url = (
        "http://memprotmd.bioch.ox.ac.uk/data/memprotmd/simulations/" + pdb_code + "_default_dppc/files/run/at.zip"
    )
    response = requests.get(url)

    open(output_file, "wb").write(response.content)

    fu.log("Saving output %s file" % (output_file), out_log, global_log)


def check_mandatory_property(property, name, out_log, classname):
    """Checks mandatory properties"""

    if not property:
        fu.log(classname + ": Unexisting %s property, exiting" % name, out_log)
        raise SystemExit(classname + ": Unexisting %s property" % name)
    return property


def check_uniprot_code(code, out_log, classname):
    """Checks uniprot code"""

    pattern = re.compile(
        (r"[OPQ][0-9][A-Z0-9]{3}[0-9]|[A-NR-Z][0-9]([A-Z][A-Z0-9]{2}[0-9]){1,2}")
    )

    if not pattern.match(code):
        fu.log(classname + ": Incorrect uniprot code for %s" % code, out_log)
        raise SystemExit(classname + ": Incorrect uniprot code for %s" % code)

    return True



#!/usr/bin/env python

"""PdbClusterZip Module"""

import os
from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import (
    check_mandatory_property,
    check_output_path,
    download_pdb,
    get_cluster_pdb_codes,
    write_pdb,
)


class PdbClusterZip(BiobbObject):
    """
    | biobb_io PdbClusterZip
    | This class is a wrapper for downloading a PDB cluster from the Protein Data Bank.
    | Wrapper for the `Protein Data Bank in Europe <https://www.ebi.ac.uk/pdbe/>`_, the `Protein Data Bank <https://www.rcsb.org/>`_ and the `MMB PDB mirror <https://mdb-login.bsc.es/api/>`_ for downloading a PDB cluster.

    Args:
        output_pdb_zip_path (str): Path to the ZIP file containing the output PDB files. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_pdb_cluster.zip>`_. Accepted formats: zip (edam:format_3987).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB code.
            * **filter** (*str*) - (["ATOM", "MODEL", "ENDMDL"]) Array of groups to be kept. If value is None or False no filter will be applied. All the possible values are defined in the official PDB specification (http://www.wwpdb.org/documentation/file-format-content/format33/v3.3.html)
            * **cluster** (*int*) - (90) Sequence Similarity Cutoff. Values: 50 (structures having less than 50% sequence identity to each other), 70 (structures having less than 70% sequence identity to each other), 90 (structures having less than 90% sequence identity to each other), 95 (structures having less than 95% sequence identity to each other).
            * **api_id** (*str*) - ("pdbe") Identifier of the PDB REST API from which the PDB structure will be downloaded. Values: pdbe (`PDB in Europe REST API <https://www.ebi.ac.uk/pdbe/pdbe-rest-api>`_), pdb (`RCSB PDB REST API <https://data.rcsb.org/>`_), mmb (`MMB PDB mirror API <https://mdb-login.bsc.es/api/>`_).
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.pdb_cluster_zip import pdb_cluster_zip
            prop = {
                'pdb_code': '2VGB',
                'filter': ['ATOM', 'MODEL', 'ENDMDL'],
                'cluster': 90,
                'api_id': 'pdbe'
            }
            pdb_cluster_zip(output_pdb_zip_path='/path/to/newStructures.zip',
                            properties=prop)

    Info:
        * wrapped_software:
            * name: Protein Data Bank
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_pdb_zip_path, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_pdb_zip_path": output_pdb_zip_path}}

        # Properties specific for BB
        self.api_id = properties.get("api_id", "pdbe")
        self.pdb_code = properties.get("pdb_code", None)
        self.filter = properties.get("filter", ["ATOM", "MODEL", "ENDMDL"])
        self.cluster = properties.get("cluster", 90)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_pdb_zip_path = check_output_path(
            self.io_dict["out"]["output_pdb_zip_path"],
            "output_pdb_zip_path",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`PdbClusterZip <api.pdb_cluster_zip.PdbClusterZip>` api.pdb_cluster_zip.PdbClusterZip object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        check_mandatory_property(
            self.pdb_code, "pdb_code", self.out_log, self.__class__.__name__
        )

        self.pdb_code = self.pdb_code.strip().lower()

        file_list = []
        # Downloading PDB_files
        pdb_code_list = get_cluster_pdb_codes(
            pdb_code=self.pdb_code,
            cluster=self.cluster,
            out_log=self.out_log,
            global_log=self.global_log,
        )
        unique_dir = fu.create_unique_dir()
        for pdb_code in pdb_code_list:
            pdb_file = os.path.join(unique_dir, pdb_code + ".pdb")
            pdb_string = download_pdb(
                pdb_code=pdb_code,
                api_id=self.api_id,
                out_log=self.out_log,
                global_log=self.global_log,
            )
            write_pdb(pdb_string, pdb_file, self.filter, self.out_log, self.global_log)
            file_list.append(os.path.abspath(pdb_file))

        # Zipping files
        fu.log("Zipping the pdb files to: %s" % self.output_pdb_zip_path)
        fu.zip_list(self.output_pdb_zip_path, file_list, out_log=self.out_log)

        self.tmp_files.append(unique_dir)
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def pdb_cluster_zip(
    output_pdb_zip_path: str, properties: Optional[dict] = None, **kwargs
) -> int:
    """Execute the :class:`PdbClusterZip <api.pdb_cluster_zip.PdbClusterZip>` class and
    execute the :meth:`launch() <api.pdb_cluster_zip.PdbClusterZip.launch>` method."""
    return PdbClusterZip(**dict(locals())).launch()


pdb_cluster_zip.__doc__ = PdbClusterZip.__doc__
main = PdbClusterZip.get_main(pdb_cluster_zip, "Wrapper for the Protein Data Bank in Europe (https://www.ebi.ac.uk/pdbe/), the Protein Data Bank (https://www.rcsb.org/) and the MMB PDB mirror (https://mdb-login.bsc.es/api/) for downloading a PDB cluster.")

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""Module containing the MemProtMDSimSearch class and the command line interface."""

from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import check_output_path, get_memprotmd_sim_search, write_json


class MemProtMDSimSearch(BiobbObject):
    """
    | biobb_io MemProtMDSimSearch
    | This class is a wrapper of the MemProtMD to perform advanced searches in the MemProtMD DB using its REST API.
    | Wrapper for the `MemProtMD DB REST API <http://memprotmd.bioch.ox.ac.uk/>`_ to perform advanced searches.

    Args:
        output_simulations (str): Path to the output JSON file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_sim_search_schema_validator.json>`_. Accepted formats: json (edam:format_3464).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **collection_name** (*str*) - ("refs") Name of the collection to query.
            * **keyword** (*str*) - (None) String to search for in the database metadata. Examples are families like gpcr or porin. Values: porin, outer membrane protein, membrane protein, gpcr (7-transmembrane domain receptors transducing extracellular signals into cells), ion channels, rhodopsin (The most famous GPCRs), abc, mip (Major Intrinsic Protein (MIP)/FNT superfamily: specific for the transport of water and small neutral solutes), ligand-gated (Ligand-dependent signal conversion from chemical signals to electric signals), ammonia (Regulating transepithelial ammonia secretion), mapeg (Eicosanoid and Glutathione metabolism proteins), transmembrane (Heme biosynthesis), protein, kinase (Tyrosine-protein kinases: regulate central nervous system; gene transcription and cell differentiation), glycoprotein (Expression of TCR complex), immunoglobulin (Recognition; binding and adhesion process of cells), integrin (Bridges for cell-cell and cell-extracellular matrix interaction), bnip3 (BNip3 protein family: protect cell from apoptosis), bcl-2 (Regulating cell-death; either induce apoptotic or inhibit apoptosis), atpase (ATPase regulators; P-P-bond hydrolysis-driven transporter), cytochrome (Terminal oxidase enzyme in electron transfer chain), nadp (Transmembrane proteins with NAD(P)-binding Rossmann-fold domains: monoamine oxidase; deaminates norepinephrine; epinephrine; serotonin and dopamine), a4 (Amyloid beta A4 protein; involved in alzheimer's diseases), lysosome (Lysosome-associated membrane glycoprotein: specific to lysosomes; CD107), necrosis (Tumor necrosis factor recepto: binding with TNF and NGF; interacting with a variety of signal molecules; highly associated with apoptosis), oxidoreductase (DHODH; biosynthesis of orotate), ceramidase (Neutral/alkaline ceramidase: converting sphingolipid to sphingosine), dehydrogenase (Aldehyde dehydrogenase:ALDH; Oxidation of aldehydes), mitochondrial, plastid.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.memprotmd_sim_search import memprotmd_sim_search
            prop = {
                'collection_name': 'refs',
                'keyword': 'porin'
            }
            memprotmd_sim_search(output_simulations='/path/to/newSimulationSearch.json',
                                properties=prop).launch()

    Info:
        * wrapped_software:
            * name: MemProtMD DB
            * license: Creative Commons
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_simulations, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_simulations": output_simulations}}

        # Properties specific for BB
        self.collection_name = properties.get("collection_name", "refs")
        self.keyword = properties.get("keyword", None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_simulations = check_output_path(
            self.io_dict["out"]["output_simulations"],
            "output_simulations",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`MemProtMDSimSearch <api.memprotmd_sim_search.MemProtMDSimSearch>` api.memprotmd_sim_search.MemProtMDSimSearch object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        self.keyword = self.keyword.strip().lower()

        # get JSON object
        json_string = get_memprotmd_sim_search(
            self.collection_name, self.keyword, self.out_log, self.global_log
        )

        # write JSON file
        write_json(json_string, self.output_simulations, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def memprotmd_sim_search(
    output_simulations: str, properties: Optional[dict] = None, **kwargs
) -> int:
    """Execute the :class:`MemProtMDSimSearch <api.memprotmd_sim_search.MemProtMDSimSearch>` class and
    execute the :meth:`launch() <api.memprotmd_sim_search.MemProtMDSimSearch.launch>` method."""
    return MemProtMDSimSearch(**dict(locals())).launch()


memprotmd_sim_search.__doc__ = MemProtMDSimSearch.__doc__
main = MemProtMDSimSearch.get_main(memprotmd_sim_search, "Wrapper for the MemProtMD DB REST API (http://memprotmd.bioch.ox.ac.uk/) to perform advanced searches.")

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""Module containing the MemProtMDSimList class and the command line interface."""

from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger
from biobb_io.api.common import check_output_path, get_memprotmd_sim_list, write_json


class MemProtMDSimList(BiobbObject):
    """
    | biobb_io MemProtMDSimList
    | This class is a wrapper of the MemProtMD to get all available membrane-protein systems from its REST API.
    | Wrapper for the `MemProtMD DB REST API <http://memprotmd.bioch.ox.ac.uk/>`_ to get all available membrane-protein systems (simulations).

    Args:
        output_simulations (str): Path to the output JSON file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_sim_list_schema_validator.json>`_. Accepted formats: json (edam:format_3464).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.memprotmd_sim_list import memprotmd_sim_list
            prop = { }
            memprotmd_sim_list(output_simulations='/path/to/newSimulationlist.json',
                                properties=prop)

    Info:
        * wrapped_software:
            * name: MemProtMD DB
            * license: Creative Commons
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_simulations, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_simulations": output_simulations}}

        # Properties specific for BB
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_simulations = check_output_path(
            self.io_dict["out"]["output_simulations"],
            "output_simulations",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`MemProtMDSimList <api.memprotmd_sim_list.MemProtMDSimList>` api.memprotmd_sim_list.MemProtMDSimList object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        # get JSON object
        json_string = get_memprotmd_sim_list(self.out_log, self.global_log)

        # write JSON file
        write_json(json_string, self.output_simulations, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def memprotmd_sim_list(
    output_simulations: str, properties: Optional[dict] = None, **kwargs
) -> int:
    """Execute the :class:`MemProtMDSimList <api.memprotmd_sim_list.MemProtMDSimList>` class and
    execute the :meth:`launch() <api.memprotmd_sim_list.MemProtMDSimList.launch>` method."""
    return MemProtMDSimList(**dict(locals())).launch()


memprotmd_sim_list.__doc__ = MemProtMDSimList.__doc__
main = MemProtMDSimList.get_main(memprotmd_sim_list, "Wrapper for the MemProtMD DB REST API (http://memprotmd.bioch.ox.ac.uk/) to get all available membrane-protein systems (simulations).")

if __name__ == "__main__":
    main()



from . import (
    alphafold,
    api_binding_site,
    canonical_fasta,
    ideal_sdf,
    ligand,
    memprotmd_sim,
    memprotmd_sim_list,
    memprotmd_sim_search,
    mmcif,
    pdb,
    pdb_cluster_zip,
    pdb_variants,
    structure_info,
    mddb
)

name = "api"
__all__ = [
    "ligand",
    "pdb",
    "alphafold",
    "pdb_cluster_zip",
    "pdb_variants",
    "memprotmd_sim_list",
    "memprotmd_sim_search",
    "memprotmd_sim",
    "api_binding_site",
    "canonical_fasta",
    "mmcif",
    "ideal_sdf",
    "structure_info",
    "mddb"
]



#!/usr/bin/env python

"""PdbVariants Module"""

import re
import requests
from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import (
    check_mandatory_property,
    check_output_path,
    get_uniprot,
    get_variants,
)


class PdbVariants(BiobbObject):
    """
    | biobb_io PdbVariants
    | This class creates a text file containing a list of all the variants mapped to a PDB code from the corresponding UNIPROT entries.
    | Wrapper for the `UNIPROT <http://www.uniprot.org/>`_ mirror of the `MMB group REST API <https://mdb-login.bsc.es/api/>`_ for creating a list of all the variants mapped to a PDB code from the corresponding UNIPROT entries.

    Args:
        output_mutations_list_txt (str): Path to the TXT file containing an ASCII comma separated values of the mutations. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_pdb_variants.txt>`_. Accepted formats: txt (edam:format_2330).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB four letter code.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
            This is a use example of how to use the PdbVariants module from Python

            from biobb_io.api.pdb_variants import pdb_variants
            prop = {
                'pdb_code': '2VGB'
            }
            pdb_variants(output_mutations_list_txt='/path/to/newMutationslist.txt',
                        properties=prop)

    Info:
        * wrapped_software:
            * name: UNIPROT
            * license: Creative Commons
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_mutations_list_txt, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_mutations_list_txt": output_mutations_list_txt}}

        # Properties specific for BB
        self.pdb_code = properties.get("pdb_code", None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_mutations_list_txt = check_output_path(
            self.io_dict["out"]["output_mutations_list_txt"],
            "output_mutations_list_txt",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`PdbVariants <api.pdb_variants.PdbVariants>` api.pdb_variants.PdbVariants object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        check_mandatory_property(
            self.pdb_code, "pdb_code", self.out_log, self.__class__.__name__
        )

        self.pdb_code = self.pdb_code.strip().lower()

        url = "https://mdb-login.bsc.es/api"
        uniprot_id = get_uniprot(self.pdb_code, url, self.out_log, self.global_log)
        url_mapPDBRes = (
            url + "/uniprot/" + uniprot_id + "/mapPDBRes?pdbId=" + self.pdb_code
        )
        pattern = re.compile(
            (r"p.(?P<wt>[a-zA-Z]{3})(?P<resnum>\d+)(?P<mt>[a-zA-Z]{3})")
        )

        fu.log(
            "Fetching variants for uniprot_id: %s and pdb_code: %s"
            % (uniprot_id, self.pdb_code),
            self.out_log,
            self.global_log,
        )
        unfiltered_dic = requests.get(url_mapPDBRes, verify=True).json()
        if not unfiltered_dic:
            fu.log("No mutation found", self.out_log, self.global_log)
            return 1

        mapdic = requests.get(url_mapPDBRes, verify=True).json()
        mutations = []
        uniprot_var_list = get_variants(uniprot_id, url, self.out_log, self.global_log)
        for var in uniprot_var_list:
            match = pattern.match(var)
            if match:
                uni_mut = match.groupdict()
            else:
                continue
            for k in mapdic.keys():
                for fragment in mapdic[k]:
                    if (
                        int(fragment["unp_start"]) <= int(uni_mut["resnum"]) <= int(fragment["unp_end"])
                    ):
                        resnum = (
                            int(uni_mut["resnum"]) + int(fragment["pdb_start"]) - int(fragment["unp_start"])
                        )
                        mutations.append(
                            k[-1] + "." + uni_mut["wt"] + str(resnum) + uni_mut["mt"]
                        )

        fu.log(
            "Found %d mutations mapped to PDB: %s" % (len(mutations), self.pdb_code),
            self.out_log,
            self.global_log,
        )
        fu.log(
            "Writting mutations to: %s" % self.output_mutations_list_txt,
            self.out_log,
            self.global_log,
        )

        if not self.output_mutations_list_txt:
            raise ValueError("Output mutations list file path is not specified.")

        with open(self.output_mutations_list_txt, "w") as mut_file:
            mutations.sort()
            mut_file.write(",".join(mutations))

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def pdb_variants(
    output_mutations_list_txt: str, properties: Optional[dict] = None, **kwargs
) -> int:
    """Execute the :class:`PdbVariants <api.pdb_variants.PdbVariants>` class and
    execute the :meth:`launch() <api.pdb_variants.PdbVariants.launch>` method."""
    return PdbVariants(**dict(locals())).launch()


pdb_variants.__doc__ = PdbVariants.__doc__
main = PdbVariants.get_main(pdb_variants, "Wrapper for the UNIPROT (http://www.uniprot.org/) mirror of the MMB group REST API (https://mdb-login.bsc.es/api/) for creating a list of all the variants mapped to a PDB code from the corresponding UNIPROT entries.")

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""Module containing the Ligand class and the command line interface."""

from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import (
    check_mandatory_property,
    check_output_path,
    download_ligand,
    write_pdb,
)


class Ligand(BiobbObject):
    """
    | biobb_io Ligand
    | This class is a wrapper for downloading a PDB ligand from the Protein Data Bank.
    | Wrapper for the `Protein Data Bank in Europe <https://www.ebi.ac.uk/pdbe/>`_ and the `MMB PDB mirror <https://mdb-login.bsc.es/api/>`_ for downloading a single PDB ligand.

    Args:
        output_pdb_path (str): Path to the output PDB ligand file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_ligand.pdb>`_. Accepted formats: pdb (edam:format_1476).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **ligand_code** (*str*) - (None) RSCB PDB ligand code.
            * **api_id** (*str*) - ("pdbe") Identifier of the PDB REST API from which the PDB structure will be downloaded. Values: pdbe (`PDB in Europe REST API <https://www.ebi.ac.uk/pdbe/pdbe-rest-api>`_), mmb (`MMB PDB mirror API <https://mdb-login.bsc.es/api/>`_).
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.ligand import ligand
            prop = {
                'ligand_code': 'CPB',
                'api_id': 'pdbe'
            }
            ligand(output_pdb_path='/path/to/newLigand.pdb',
                    properties=prop)

    Info:
        * wrapped_software:
            * name: Protein Data Bank
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_pdb_path, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_pdb_path": output_pdb_path}}

        # Properties specific for BB
        self.api_id = properties.get("api_id", "pdbe")
        self.ligand_code = properties.get("ligand_code", None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_pdb_path = check_output_path(
            self.io_dict["out"]["output_pdb_path"],
            "output_pdb_path",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Ligand <api.ligand.Ligand>` api.ligand.Ligand object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        check_mandatory_property(
            self.ligand_code, "ligand_code", self.out_log, self.__class__.__name__
        )

        self.ligand_code = self.ligand_code.strip().lower()

        # Downloading PDB file
        pdb_string = download_ligand(
            self.ligand_code, self.api_id, self.out_log, self.global_log
        )
        write_pdb(pdb_string, self.output_pdb_path, None, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def ligand(output_pdb_path: str, properties: Optional[dict] = None, **kwargs) -> int:
    """Execute the :class:`Ligand <api.ligand.Ligand>` class and
    execute the :meth:`launch() <api.ligand.Ligand.launch>` method."""
    return Ligand(**dict(locals())).launch()


ligand.__doc__ = Ligand.__doc__
main = Ligand.get_main(ligand, "Wrapper for the Protein Data Bank in Europe (https://www.ebi.ac.uk/pdbe/) and the MMB PDB mirror (https://mdb-login.bsc.es/api/) for downloading a single PDB ligand.")

if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="biobb_io",
    version="5.2.1",
    author="Biobb developers",
    author_email="pau.andrio@bsc.es",
    description="Biobb_io is the Biobb module collection to fetch data to be consumed by the rest of the Biobb building blocks.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="Bioinformatics Workflows BioExcel Compatibility",
    url="https://github.com/bioexcel/biobb_io",
    project_urls={
        "Documentation": "http://biobb-io.readthedocs.io/en/latest/",
        "Bioexcel": "https://bioexcel.eu/",
    },
    packages=setuptools.find_packages(exclude=["docs", "test"]),
    package_data={"biobb_io": ["py.typed"]},
    install_requires=["biobb_common==5.2.0"],
    python_requires=">=3.10",
    entry_points={
        "console_scripts": [
            "alphafold = biobb_io.api.alphafold:main",
            "api_binding_site = biobb_io.api.api_binding_site:main",
            "canonical_fasta = biobb_io.api.canonical_fasta:main",
            "ideal_sdf = biobb_io.api.ideal_sdf:main",
            "ligand = biobb_io.api.ligand:main",
            "mddb = biobb_io.api.mddb:main",
            "memprotmd_sim_list = biobb_io.api.memprotmd_sim_list:main",
            "memprotmd_sim_search = biobb_io.api.memprotmd_sim_search:main",
            "memprotmd_sim = biobb_io.api.memprotmd_sim:main",
            "mmcif = biobb_io.api.mmcif:main",
            "pdb_cluster_zip = biobb_io.api.pdb_cluster_zip:main",
            "pdb_variants = biobb_io.api.pdb_variants:main",
            "pdb = biobb_io.api.pdb:main",
            "structure_info = biobb_io.api.structure_info:main",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Unix",
    ],
)



#!/usr/bin/env python

"""Module containing the AlphaFold class and the command line interface."""

from typing import Optional
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger

from biobb_io.api.common import (
    check_mandatory_property,
    check_output_path,
    check_uniprot_code,
    download_af,
    write_pdb,
)


class AlphaFold(BiobbObject):
    """
    | biobb_io AlphaFold
    | This class is a wrapper for downloading a PDB structure from the AlphaFold Protein Structure Database.
    | Wrapper for the `AlphaFold Protein Structure Database <https://alphafold.ebi.ac.uk/>`_ for downloading a single PDB structure from its corresponding Uniprot code.

    Args:
        output_pdb_path (str): Path to the output PDB file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_alphafold.pdb>`_. Accepted formats: pdb (edam:format_1476).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **uniprot_code** (*str*) - (None) Uniprot code.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.alphafold import alphafold
            prop = {
                'uniprot_code': 'P00489'
            }
            alphafold(output_pdb_path='/path/to/newStructure.pdb',
                properties=prop)

    Info:
        * wrapped_software:
            * name: AlphaFold Protein Structure Database
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_pdb_path, properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {"out": {"output_pdb_path": output_pdb_path}}

        # Properties specific for BB
        self.uniprot_code = properties.get("uniprot_code", None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """Checks all the input/output paths and parameters"""
        self.output_pdb_path = check_output_path(
            self.io_dict["out"]["output_pdb_path"],
            "output_pdb_path",
            False,
            out_log,
            self.__class__.__name__,
        )

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`AlphaFold <api.alphafold.AlphaFold>` api.alphafold.AlphaFold object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0

        check_mandatory_property(
            self.uniprot_code, "uniprot_code", self.out_log, self.__class__.__name__
        )

        self.uniprot_code = self.uniprot_code.strip().upper()

        check_uniprot_code(self.uniprot_code, self.out_log, self.__class__.__name__)

        # Downloading PDB file
        pdb_string = download_af(
            self.uniprot_code, self.out_log, self.global_log, self.__class__.__name__
        )
        write_pdb(pdb_string, self.output_pdb_path, None, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0


def alphafold(output_pdb_path: str, properties: Optional[dict] = None, **kwargs) -> int:
    """Execute the :class:`AlphaFold <api.alphafold.AlphaFold>` class and
    execute the :meth:`launch() <api.alphafold.AlphaFold.launch>` method."""
    return AlphaFold(**dict(locals())).launch()


alphafold.__doc__ = AlphaFold.__doc__
main = AlphaFold.get_main(alphafold, "This class is a wrapper for downloading a PDB structure from the Protein Data Bank.")

if __name__ == "__main__":
    main()
